#!/usr/bin/env python3
# coding: utf-8

import os
import re
import sys
from collections import defaultdict

from joker.cast.iterative import split
from joker.stream.shell import ShellStream
from joker.stream.utils import checksum

from joker.superuser import utils


def _thresh(size, minsize):
    if size >= minsize:
        return size


def get_sizefunc(minsize):
    sizefunc = utils.silent_function(os.path.getsize)
    if not minsize:
        return sizefunc
    return lambda p: _thresh(sizefunc(p), minsize)


@utils.silent_function
def chksum(path):
    return checksum(path).digest()


@utils.silent_function
def chksum_head(path, length=1024):
    return checksum(path, length=length).digest()


class ChecksumCache(object):
    def __init__(self, path):
        data = {}
        for line in open(path):
            v, k = line.split(maxsplit=1)
            data[k] = v
        self.data = data

    @utils.silent_function
    def __call__(self, path):
        data = self.data
        try:
            return data[path]
        except LookupError:
            return checksum(path).digest()

    @classmethod
    def opt(cls, path):
        if not path:
            return chksum
        try:
            return cls(path)
        except Exception:
            return chksum


def _plural(grp):
    if len(grp) > 1:
        return grp


def _subgroup(groups, kfunc, gfunc):
    """
    separate each group into sub-groups
    :param groups: a list of lists
    :param kfunc: kfunc(path) => a differentiating property of a file,
        or None to ignore that file
    :param gfunc: gfunc(group) => a modified group,
        or None to ignore that group
    :return: a list of lists (sub-groups)
    """
    subgrps = []
    for grp in groups:
        aux = defaultdict(list)
        for path in grp:
            k = kfunc(path)
            if k is not None:
                aux[k].append(path)
        batch = (gfunc(g) for g in aux.values())
        batch = (g for g in batch if g is not None)
        subgrps.extend(batch)
    return subgrps


def find_duplicates(dirs, minsize=0, bold=False, chksumfile=None):
    all_files = []
    for path in dirs:
        all_files.extend(utils.find_regular_files(path))
    sizefunc = get_sizefunc(minsize)
    grps = _subgroup([all_files], sizefunc, _plural)
    grps = _subgroup(grps, chksum_head, _plural)
    if not bold:
        chksum_func = ChecksumCache.opt(chksumfile)
        grps = _subgroup(grps, chksum_func, _plural)
    for grp in grps:
        grp.sort()
    return grps


def parse_fdupes_output(path):
    with ShellStream.open(path).snl() as stream:
        for tup in split(stream, lambda x: not x):
            elements = tup[-1]
            if elements:
                yield elements


def check_existence(groups):
    for grp in groups:
        paths = [p for p in grp if os.path.isfile(p)]
        if len(paths) > 1:
            paths.sort(reverse=True)
            yield paths


def search_pattern(groups, pattern):
    for grp in groups:
        for path in grp:
            if re.search(pattern, path):
                yield grp
                break


def _make_pairs(paths, protect_regex):
    pairs = []
    for i, path in enumerate(paths):
        if protect_regex.match(path):
            pairs.append((0, path))
        else:
            pairs.append((1 + i, path))
    pairs.sort()
    return pairs


def prioritize(groups, protect=None):
    if protect:
        regex = re.compile(protect)
        for grp in groups:
            yield _make_pairs(grp, regex)
    else:
        for grp in groups:
            yield [(1 + i, p) for i, p in enumerate(grp)]


def _make_script(pairs):
    from shlex import quote
    pairs = list(pairs)
    if not any(p[0] for p in pairs):
        return
    path = pairs[0][1]
    print('#', quote(path))
    for prio, path in pairs[1:]:
        if prio:
            print('rm -f', quote(path))
        else:
            print('#', quote(path))
    print()


def _parse_args(prog, args):
    import argparse
    desc = 'find duplicating files recursively'
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    aa = pr.add_argument

    aa('-B', '--bold', action='store_true',
       help='skip full-size checksum comparison')

    aa('-c', '--chksumfile',
       help='a text file as generated by md5sum utility')

    aa('-s', '--size', type=int, default=1,
       help='minimum size for consideration')

    aa('-p', '--pattern', metavar='regex',
       help='select a subset of duplicating groups')

    aa('-P', '--protect', metavar='regex',
       help='prevent certain file from being deleted')

    aa('-f', '--fdupes', help='an fdupes output file')
    aa('dirs', metavar='DIR', nargs='*', help='target directory')
    return pr.parse_args(args)


def run(prog, args):
    ns = _parse_args(prog, args)
    if ns.fdupes:
        if ns.dirs:
            msg = 'conflicting arguments: -f/--fdupes and dirs'
            print(msg, file=sys.stderr)
        groups = parse_fdupes_output(ns.path)
        groups = check_existence(groups)
    else:
        groups = find_duplicates(ns.dirs, minsize=ns.size,
                                 bold=ns.bold, chksumfile=ns.chksumfile)
    if ns.pattern:
        groups = search_pattern(groups, ns.pattern)
    p_groups = prioritize(groups, ns.protect)
    for pgrp in p_groups:
        _make_script(pgrp)



#!/usr/bin/env python3
# coding: utf-8

import os
from functools import wraps


def under_asset_dir(*paths):
    from joker.superuser.environ import JokerInterface
    ji = JokerInterface()
    return ji.under_package_dir('asset', *paths)


def load_config(name):
    import yaml
    from joker.superuser.environ import JokerInterface
    ji = JokerInterface
    path = ji.under_joker_subdir(name)
    if os.path.isfile(path):
        return yaml.safe_load(open(path))


def report(data, name):
    import json
    from joker.superuser.environ import JokerInterface
    ji = JokerInterface()
    if not name.endswith('.json'):
        name += '.json'
    path = ji.under_joker_subdir(name, mkdirs=True)
    with open(path, 'w') as fout:
        fout.write(json.dumps(data))


def silent_function(func):
    """Do not report any error"""

    @wraps(func)
    def sfunc(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except BaseException:
            pass

    return sfunc


def startswith(s, *prefixes):
    for p in prefixes:
        if s.startswith(p):
            return True
    return False


def find_regular_files(dirpath, **kwargs):
    for root, dirs, files in os.walk(dirpath, **kwargs):
        for name in files:
            path = os.path.join(root, name)
            if os.path.isfile(path):
                yield path


def check_filesys_case_sensitivity(dirname='.'):
    from tempfile import TemporaryDirectory
    with TemporaryDirectory(prefix='.sus-tmp', dir=dirname) as tmp:
        return not os.path.exists(tmp.upper())



#!/usr/bin/env python3
# coding: utf-8

import sys


def hexify(path: str):
    with open(path, 'rb') as fin, open(path + '.hex', 'w') as fout:
        chunk = fin.read(32)
        while chunk:
            print(chunk.hex())
            print(chunk.hex(), file=fout)
            chunk = fin.read(32)


def _run():
    for path in sys.argv[1:]:
        hexify(path)


if __name__ == '__main__':
    _run()



#!/usr/bin/env python3
# coding: utf-8

from __future__ import print_function

import sys

codename_of_vers = {
    '4': 'etch',
    '5': 'lenny',
    '6': 'squeeze',
    '7': 'wheezy',
    '8': 'jessie',
    '9': 'stretch',
    '10': 'buster',
    '11': 'bullseye',
    '12': 'bookworm',
    '13': 'trixie',
    '4.10': 'warty',
    '5.04': 'hoary',
    '5.10': 'breezy',
    '6.06': 'dapper',
    '6.10': 'edgy',
    '7.04': 'feisty',
    '7.10': 'gutsy',
    '8.04': 'hardy',
    '8.10': 'intrepid',
    '9.04': 'jaunty',
    '9.10': 'karmic',
    '10.04': 'lucid',
    '10.10': 'maverick',
    '11.04': 'natty',
    '11.10': 'oneiric',
    '12.04': 'precise',
    '12.10': 'quantal',
    '13.04': 'raring',
    '13.10': 'saucy',
    '14.04': 'trusty',
    '14.10': 'utopic',
    '15.04': 'vivid',
    '15.10': 'wily',
    '16.04': 'xenial',
    '16.10': 'yakkety',
    '17.04': 'zesty',
    '17.10': 'artful',
    '18.04': 'bionic',
    '18.10': 'cosmic',
    '19.04': 'disco',
    '19.10': 'eoan',
    '20.04': 'focal',
    '20.10': 'groovy',
}

ver_of_codenames = {v: k for k, v in codename_of_vers.items()}

apt_debian = """\
deb http{s}://{host}/debian/ {codename} main contrib non-free
deb http{s}://{host}/debian/ {codename}-updates main contrib non-free
deb http{s}://{host}/debian/ {codename}-backports main contrib non-free
deb http{s}://{host}/debian-security/ {codename}/updates main contrib non-free
"""

apt_ubuntu = """\
deb http{s}://{host}/ubuntu/ {codename} main restricted universe multiverse
deb http{s}://{host}/ubuntu/ {codename}-security main restricted universe multiverse
deb http{s}://{host}/ubuntu/ {codename}-updates main restricted universe multiverse
deb http{s}://{host}/ubuntu/ {codename}-backports main restricted universe multiverse
"""


def fmt_sources_file(host: str, version: str, https: bool):
    if version in codename_of_vers:
        v = version
        c = codename_of_vers[version]
    elif version in ver_of_codenames:
        v = ver_of_codenames[version]
        c = version
    else:
        raise ValueError('unknown version {}'.format(version))
    if v.isdecimal():
        tmpl = apt_debian
    else:
        tmpl = apt_ubuntu
    kw = {
        's': 's' if https else '',
        'host': host, 'codename': c,
    }
    return tmpl.format(**kw)


def run(prog, args):
    import argparse
    desc = 'Generate an apt-get sources.list file'
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    add = pr.add_argument
    add('-H', '--host', required=True,
        help='e.g. mirror.example.com')
    add('-v', '--version', required=True,
        help='codename or version numbers')
    add('-s', '--https', action='store_true',
        help='use https:// instead of http://')
    ns = pr.parse_args(args)
    try:
        text = fmt_sources_file(**vars(ns))
    except ValueError as e:
        print('error:', e, file=sys.stderr)
        return
    print(text)



#!/usr/bin/env python3
# coding: utf-8

import logging
import os
import re
import sys
from os.path import join, relpath

from joker.superuser.environ import JokerInterface
from joker.superuser.utils import under_asset_dir
from joker.textmanip.tabular import format_help_section

ji = JokerInterface()
_logger = logging.Logger(__name__)

_pkg_init = """
__version__ = "0.0.0"
if __name__ == "__main__":
    print(__version__)
"""

_nsp_inits = {
    'pkg_resources': '__import__("pkg_resources").declare_namespace(__name__)',
    'pkgutil': '__path__ = __import__("pkgutil").extend_path(__path__, __name__)',
    'native': None,
}

_nspinits = {
    'pkg_resources': '__import__("pkg_resources").declare_namespace(__name__)',
    'pkgutil': '__path__ = __import__("pkgutil").extend_path(__path__, __name__)',
}


def _vk_join(d):
    return {k: join(v, k) for k, v in d.items()}


class ProjectDirectoryMaker(object):
    def __init__(self, nsp, pkg):
        """
        :param nsp: namespace
        :param pkg: package name
        """
        self.nsp = nsp.strip()
        self.pkg = pkg.strip()
        if self.nsp:
            self.names = [self.nsp, self.pkg]
        else:
            self.names = [self.pkg]

        # self.dot_name = '.'.join(names)
        # self.hyf_name = '-'.join(names)
        # self.uns_name = '_'.join(names)

        self.package_name = '.'.join(self.names)
        self.project_name = '-'.join(self.names)
        self.identifier = '_'.join(self.names)
        self.common_dirs = {
            'docs': self.under_project_dir('docs'),
            'test': self.under_project_dir('tests'),
            'templates': self.under_package_dir('templates'),
        }
        self.common_files = _vk_join({
            '__init__.py': self.under_package_dir(),
            'requirements.txt': self.project_name,
            'MANIFEST.in': self.project_name,
            '.gitignore': self.project_name,
            'setup.py': self.project_name,
        })

    @classmethod
    def parse(cls, name):
        """
        :param name: e.g. "mypkg", "mynsp.mypkg"
        :return: a ProjectDirectoryMaker instance
        """
        mat = re.match(r'([_A-Za-z]\w*\.|)([_A-Za-z]\w*)', name)
        if not mat:
            raise ValueError('invalid pakcage name: ' + repr(name))
        return cls(mat.group(1)[:-1] or '', mat.group(2))

    def under_project_dir(self, *paths):
        return join(self.project_name, *paths)

    def under_package_dir(self, *paths):
        parts = [self.project_name] + self.names + list(paths)
        return join(*parts)

    def locate(self, name):
        if name.endswith('/'):
            return self.common_dirs.get(name[:-1])
        return self.common_files.get(name)

    def make_dirs(self):
        for path in self.common_dirs.values():
            os.makedirs(path, exist_ok=True)
        for path in self.common_files.values():
            open(path, 'a').close()

    def sub(self, text):
        text = text.replace('__sus_hyf_name', self.project_name)
        text = text.replace('__sus_dot_name', self.package_name)
        text = text.replace('__sus_uns_name', self.identifier)
        text = text.replace('__sus_nsp', self.nsp)
        text = text.replace('__sus_pkg', self.pkg)
        return text

    def _gettext_setup(self, template_path):
        template_path = template_path or under_asset_dir('setup.txt')
        code = open(template_path).read()
        return self.sub(code)

    def gettext_setup(self, template_path=None):
        jinja2 = ji.jinja2_env
        tpl = jinja2.get_template('setup.py.jinja2')
        return tpl.render(self.__dict__)

    def gettext_manifest(self):
        # CAUTION: not self.common_dirs['templates']!
        # relative to self.under_proj_dir()!
        return os.linesep.join([
            'include requirements.txt',
            'exclude {}/*'.format(
                relpath(self.common_dirs['test'], self.project_name),
            ),
            'recursive-include {} *.html'.format(
                relpath(self.common_dirs['templates'], self.project_name),
            )
        ]) + os.linesep

    def write(self, name, content):
        path = self.common_files.get(name)
        if path and content:
            with open(path, 'a') as fout:
                fout.write(content + os.linesep)

    def write_nspinit(self, style):
        content = _nspinits.get(style)
        if not content:
            return
        path = self.under_project_dir(self.nsp, '__init__.py')
        with open(path, 'a') as fout:
            fout.write(content)

    def write_version_variable(self):
        self.write('__init__.py', "__version__ = '0.0.1'")

    def write_requirements(self, lines):
        self.write('requirements.txt', os.linesep.join(lines))

    def write_setup(self, template_path=''):
        self.write('setup.py', self.gettext_setup(template_path))

    def write_manifest(self):
        self.write('MANIFEST.in', self.gettext_manifest())

    def write_gitignore(self, path=''):
        path = path or under_asset_dir('gitignore.txt')
        self.write('.gitignore', open(path).read())


def make_project(name, alt, setup, gitignore, require, nsp_style):
    if alt == 'nspinit':
        print(_nspinits.get(nsp_style, ''))
        return
    if alt == 'gitignore':
        print(open(under_asset_dir('gitignore.txt')).read())
        return

    mkr = ProjectDirectoryMaker.parse(name)
    if alt == 'sub':
        print(mkr.sub(sys.stdin.read()))
        return
    if alt == 'setup':
        print(mkr.gettext_setup(setup))
        return
    if alt == 'manifest':
        print(mkr.gettext_manifest())
        return

    try:
        # TODO
        return print(mkr.locate(alt or '') + '')
    except TypeError:
        pass

    if alt:
        raise ValueError('invalid action: ' + repr(alt))

    mkr.make_dirs()
    mkr.write_setup(setup)
    mkr.write_gitignore(gitignore)
    mkr.write_requirements(require or [])
    mkr.write_manifest()
    mkr.write_nspinit(nsp_style)
    mkr.write_version_variable()


def run(prog=None, args=None):
    import argparse
    desc = 'Generate a python project structure'
    # s = ' (case sensitive)'
    epilog = '\n'.join([
        format_help_section('Namespace package styles', _nspinits),
        'About namespace packages:',
        '  https://packaging.python.org/guides/packaging-namespace-packages/',
    ])
    parser = argparse.ArgumentParser(
        prog=prog, description=desc, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    add = parser.add_argument

    add('-a', '--alt',
        choices=['nspinit', 'gitignore', 'sub', 'setup', 'manifest'],
        help='alternative action')

    # https://packaging.python.org/guides/packaging-namespace-packages/
    add('-n', '--nsp-style', choices=list(_nspinits),
        default='native', help='namespace package style')

    add('-s', '--setup', metavar='template_setup.py',
        help='path to a custom setup.py template')

    add('-g', '--gitignore', metavar='template_gitignore.txt',
        help='path to a custom .gitignore template')

    add('-r', '--require', action='append', metavar='PACKAGE',
        help='example: -r six -r numpy>=1.0.0')

    add('name', metavar='[NAMESPACE.]PACKAGE_NAME[%QUERY]')
    ns = parser.parse_args(args)

    try:
        make_project(**vars(ns))
    except ValueError as e:
        print(str(e), file=sys.stderr)



#!/usr/bin/env python3
# coding: utf-8

import glob
import os
import re

import json5
from joker.interfaces.repos import Repository


def _fmt_error_line(colno: int, errmsg: str):
    leading_whitespaces = ' ' * (colno - 3)
    underline = '^' * 6
    return f'{leading_whitespaces}{underline} <- {errmsg}'


def _parse_error_msg(errmsg: str, filename: str):
    mat = re.search(r':(?P<lineno>\d+).* column (?P<colno>\d+)', errmsg)
    return (
        int(mat.groupdict()['lineno']),
        int(mat.groupdict()['colno']),
        errmsg.replace('<string>', filename),
    )


def _check_json5(path: str):
    lines = open(path)
    lines = [s.rstrip() for s in lines]
    try:
        _ = json5.loads(os.linesep.join(lines))
        print('OK', path, sep='\t')
    except ValueError as err:
        lineno, colno, errmsg = _parse_error_msg(str(err), path)
        lines.insert(lineno, _fmt_error_line(colno, errmsg))
        for line in lines:
            print(line)
        print('ERR', path, sep='\t')


def check_json5(path: str):
    try:
        _ = json5.load(open(path))
        print('OK', path, sep='\t')
    except ValueError:
        print('ERR', path, sep='\t')


def _glob_multiple(*patterns):
    paths = []
    for pat in patterns:
        paths.extend(glob.glob(pat))
    return paths


def check_json5_files():
    paths = _glob_multiple('*.json', '*.json5', '*/*.json', '*/*.json5')
    for path in paths:
        check_json5(path)


def pull_all():
    repos = Repository.find('.')
    for repo in repos:
        repo.pull()



#!/usr/bin/env python3
# coding: utf-8

import argparse
import fnmatch
import os
import sys


def remove_empty_dirs(rootdir, dry=False):
    for directory, dirnames, filenames in os.walk(rootdir, topdown=False):
        if not dirnames and not filenames:
            if not dry:
                os.rmdir(directory)
                print('REMOVED:', directory, file=sys.stderr)
            else:
                print('NOT REMOVED:', directory, file=sys.stderr)


def _check(dir_, name, patterns, empty):
    path = os.path.join(dir_, name)
    if empty and os.path.getsize(path) == 0:
        return path
    for patt in patterns:
        if fnmatch.fnmatch(name, patt):
            return path


def remove_files(rootdir, patterns, empty=False, dry=False):
    _size = os.path.getsize
    _join = os.path.join
    for dir_, dirnames, filenames in os.walk(rootdir):
        for name in filenames:
            path = _check(dir_, name, patterns, empty)
            if not path:
                continue
            if not dry:
                os.remove(path)
                print('REMOVED:', path, file=sys.stderr)
            else:
                print('NOT REMOVED:', path, file=sys.stderr)


_all_presets = {
    'pyc': ['*.pyc'],
    'log': ["*.log"],
    'win': ['desktop.ini', 'thumbs.db'],
    'mac': ['.DS_Store'],
    # 'mauve': ['mauve*'],
    'blast': [
        '*.phr', '*.pin', '*.psq',
        '*.nhr', '*.nin', '*.nsq',
        "*.nsd", "*.nsi", 'formatdb.log',
    ],
}


def _format_presets_list():
    parts = ['available presets:']
    width = max(len(name) for name in _all_presets)
    for name, patterns in _all_presets.items():
        s1 = (name + ':').ljust(width + 3)
        s2 = ', '.join(patterns)
        parts.append(s1 + s2)
    return '\n  '.join(parts)


def run(prog=None, args=None):
    desc = 'Remove (semi-)empty directories'
    pr = argparse.ArgumentParser(
        prog=prog, description=desc,
        epilog=_format_presets_list(),
        formatter_class=argparse.RawDescriptionHelpFormatter)

    pr.add_argument('-d', '--dry', action='store_true',
                    help='just show files to be removed')

    pr.add_argument('-a', '--all-presets',
                    action='store_true', help='use all presets')

    pr.add_argument('-e', '--empty', action='store_true',
                    help='remove all 0 byte files, regardless filename')

    pr.add_argument('target', metavar='dir', help='target directory')

    pr.add_argument('patterns', nargs='*', metavar='pattern',
                    help='preset names or filename patterns')

    ns = pr.parse_args(args)
    name_patterns = []
    if ns.all_presets:
        name_patterns = sum(_all_presets.values(), name_patterns)

    for pat in ns.patterns:
        name_patterns.extend(_all_presets.get(pat, [pat]))
    if name_patterns or ns.empty:
        remove_files(ns.target, set(name_patterns), ns.empty, ns.dry)
    remove_empty_dirs(ns.target, ns.dry)



#!/usr/bin/env python3
# coding: utf-8

import os.path
import platform
import sys

from volkanic.utils import under_home_dir


def under_chrome_default_dir(*paths):
    # https://stackoverflow.com/a/13874620/2925169
    # https://newbedev.com/how-can-i-view-archived-google-chrome-history
    # -i-e-history-older-than-three-months

    if sys.platform == 'darwin':
        return under_home_dir(
            'Library/Application Support/Google/Chrome/Default', *paths,
        )
    elif sys.platform in ['linux', 'linux2']:
        return under_home_dir('.config/google-chrome/Default', *paths)
    elif sys.platform in ['win32', 'cygwin', 'msys']:
        if platform.win32_ver()[0] in ['XP', '2000']:
            return under_home_dir(
                r'Local Settings\Application Data',
                r'Google\Chrome\User Data\Default',
                *paths,
            )
        else:
            return under_home_dir(
                r'\AppData\Local\Google\Chrome\User Data\Default',
                *paths,
            )
    else:
        raise RuntimeError('current platform is not supported')


def get_chrome_history_db_files():
    paths = [
        under_chrome_default_dir('History'),
        under_chrome_default_dir('Archived History'),
    ]
    return [os.path.exists(p) for p in paths]



#!/usr/bin/env python3
# coding: utf-8

import os
import re

from joker.cast.syntax import printerr

_template = r"""#!/usr/bin/env python3
# coding: utf-8
import re, sys; from {mod} import {call}
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit({call}())
"""

_regex = re.compile(r'(?P<cmd>\w+)=(?P<mod>\w[\w.]*\w):(?P<call>\w+)$')


def make_entrypoint_script(spec, targetdir=None):
    spec = ''.join(spec.split())
    mat = _regex.match(spec)
    if not mat:
        raise ValueError('bad spec "{}"'.format(spec))
    params = mat.groupdict()
    text = _template.format(**params)
    if not targetdir:
        return print(text)
    path = os.path.join(targetdir, params.get('cmd'))
    if os.path.exists(path):
        raise FileExistsError(path)
    with open(path, 'w') as fout:
        fout.write(text)
    os.chmod(path, int('754', 8))


class Spy(object):
    def __init__(self):
        self.args = None
        self.kwargs = None

    def __call__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def load_setup_file(self, path):
        import setuptools
        path = os.path.abspath(path)
        cwd = os.getcwd()
        setup = setuptools.setup
        try:
            os.chdir(os.path.dirname(path))
            setuptools.setup = self
            code = open(path).read()
            exec(code, {'__file__': path})
        finally:
            os.chdir(cwd)
            setuptools.setup = setup


def inspect_setuppy(path):
    path = os.path.abspath(path)
    if os.path.isdir(path):
        path = os.path.join(path, 'setup.py')
    if not os.path.isfile(path):
        raise FileNotFoundError(path)
    spy = Spy()
    spy.load_setup_file(path)
    return spy.kwargs['entry_points']['console_scripts'][0]


def run(prog, args):
    import argparse
    desc = 'generate entrypoint script from setup.py'
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    aa = pr.add_argument
    aa('-d', '--dir', help='output dir')
    aa('path', help='path to a setup.py or its containing dir')
    ns = pr.parse_args(args)
    try:
        spec = inspect_setuppy(ns.path)
        make_entrypoint_script(spec, ns.dir)
    except Exception as e:
        printerr(e)
